use std::any::type_name;

use shred::{Fetch, FetchMut, Resource};
use shrev::{Event, EventChannel, ReaderId};
use specs::World;

use crate::Error;

pub trait WorldHelper {
    fn resource<R>(&self) -> Result<Fetch<R>, Error>
    where
        R: Resource;

    fn resource_mut<R>(&self) -> Result<FetchMut<R>, Error>
    where
        R: Resource;

    fn register_event_reader<E>(&self) -> Result<ReaderId<E>, Error>
    where
        E: Event;
}

impl WorldHelper for World {
    fn resource<R>(&self) -> Result<Fetch<R>, Error>
    where
        R: Resource,
    {
        self.try_fetch::<R>()
            .ok_or_else(|| Error::ResourceNotRegistered(type_name::<R>()))
    }

    fn resource_mut<R>(&self) -> Result<FetchMut<R>, Error>
    where
        R: Resource,
    {
        self.try_fetch_mut::<R>()
            .ok_or_else(|| Error::ResourceNotRegistered(type_name::<R>()))
    }

    fn register_event_reader<E>(&self) -> Result<ReaderId<E>, Error>
    where
        E: Event,
    {
        Ok(self.resource_mut::<EventChannel<E>>()?.register_reader())
    }
}
